/*
 * JarResourceURLConnection
 *
 * Copyright (c) 2005 Eugene Matyushkin (E-mail: skipy@mail.ru)
 *
 * License agreement:
 *
 * 1. This code is published AS IS. Author is not responsible for any damage that can be
 *    caused by any application that uses this code.
 * 2. Author does not give a garantee, that this code is error free.
 * 3. This code can be used in NON-COMMERCIAL applications AS IS without any special
 *    permission from author.
 * 4. This code can be modified without any special permission from author IF AND ONLY IF
 *    this license agreement will remain unchanged.
 * 5. SPECIAL PERMISSION for this code usage in COMMERCIAL application SHOULD be obtained
 *    from author.
 */
package ru.skipy.net.protocol.jarres;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;

/**
 * URLConnection implementation for <code>jarres</code> protocol.
 *
 * @author Eugene Matyushkin
 * @version 1.0
 */
public class JarResourceURLConnection extends URLConnection {

    /**
     * Path to the jar resource
     */
    private String resourcePath;

    /**
     * Connection flag
     */
    private boolean connected = false;

    /**
     * Stream to read resource
     */
    private InputStream is = null;

    /**
     * Constructs a <code>URLConnection</code> to the specified URL.
     *
     * @param url URL to connect to.
     */
    JarResourceURLConnection(URL url) {
        super(url);
        resourcePath = url.getPath();
        super.setDoOutput(false);
    }

    /**
     * Connects to the resource.
     *
     * @throws IOException if IO error occures while connecting
     */
    public synchronized void connect() throws IOException {
        if (connected) return;
        byte data[] = readData();
        is = (data == null) ? null : new ByteArrayInputStream(data);
        connected = true;
    }

    /**
     * Reads resource data. Returns byte array that contains resource content.
     *
     * @return byte array
     *
     * @throws IOException if some IO error occures while reading resource data.
     */
    private byte[] readData() throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        InputStream resourceStream = getClass().getResourceAsStream(resourcePath);
        if (resourceStream == null)
            return null;
        DataInputStream dis = new DataInputStream(resourceStream);
        byte[] buffer = new byte[16384];
        while (true) {
            int readed = dis.read(buffer);
            if (readed == -1)
                break;
            if (readed > 0)
                baos.write(buffer, 0, readed);
        }
        return baos.toByteArray();
    }

    /**
     * Returns InputStream, from which resource content can be read.
     *
     * @return input stream, that contains resource content
     *
     * @throws FileNotFoundException if resource is not found
     * @throws IOException           if some general IO error occures
     */
    public synchronized InputStream getInputStream() throws IOException {
        connect();
        if (is == null) throw new FileNotFoundException(resourcePath);
        return is;
    }

    /**
     * Output is not permitted for this type of connection, thus this method just overrides
     * existing one an does nothing.
     *
     * @param dooutput parameter, that required by method signature. Means nothing
     *                 in this implementation
     */
    public void setDoOutput(boolean dooutput) {
    }

}
